;------------------------------------------------------------------------------
;	Fast Assembly MPEG Encoder
;------------------------------------------------------------------------------
; format_conv.nas:
;	Converts between various image formats (YUV9, YUV12, YUYV,...)
;
; Notes: This is quite slow. Try to avoid have to convert between formats
;------------------------------------------------------------------------------
;
;	Copyright (C) 1999-2000 Vivien Chappelier
;
; *  This program is free software; you can redistribute it and/or modify
; *  it under the terms of the GNU General Public Licensse as published by
; *  the Free Software Foundation; either version 2 of the License, or
; *  (at your option) any later version.
; *
; *  This program is distributed in the hope that it will be useful,
; *  but WITHOUT ANY WARRANTY; without even the implied warranty of
; *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; *  GNU General Public License for more details.
; *
; *  You should have received a copy of the GNU General Public License
; *  along with this program; if not, write to the Free Software
; *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;
;------------------------------------------------------------------------------

%macro cglobal 1 
          global _%1 
%define %1 _%1 
%endmacro

%macro cextern 1 
          extern _%1 
%define %1 _%1 
%endmacro

%ifdef _WIN32_
%define public cglobal
%endif

%ifdef _LINUX_
%define public global
%endif

section .data

	YUYV_maskY  dd 0FF00FF00h
		    dd 0FF00FF00h
	YUYV_maskUV dd 000FF00FFh
		    dd 000FF00FFh
	UYVY_maskY  dd 000FF00FFh
		    dd 000FF00FFh
	UYVY_maskUV dd 0FF00FF00h
		    dd 0FF00FF00h

	YUV9_offsetU	dd 0
	YUV9_offsetV	dd 0
	YUV12_offsetU	dd 0
	YUV12_offsetV	dd 0

	_test		dd 000800080h
			dd 000400040h
section .text

	public YUYV_to_YUV9
	public YUYV_to_YUV12i
	public UYVY_to_YUV12i

	; TEMP
	public TESTMEM

align 16
TESTMEM:
	push ebp
	mov ebp, esp
	push edi
	push esi
	push ebx

	mov edx, 1000
	mov eax, 0
align 16
.loop1:
	mov edi, [ebp + 8]
	mov esi, [ebp + 8]

;	mov ecx, 65536
;	rep movsd
	mov ecx, 65536/8
align 16
.loop2:
	movq mm0, [edi]
	movq mm1, [edi+8]
	movq mm2, [edi+16]
	movq mm3, [edi+24]
	movq [esi], mm0
	movq [esi+8], mm1
	movq [esi+16], mm2
	movq [esi+24], mm3
	dec ecx
	jnz .loop2

	dec edx
	jnz .loop1

	emms

	pop ebx
	pop esi
	pop edi
	pop ebp
	ret	
		
;	Read the Y component from an 8x2 pixel block and write it down
;	Assumes:
;		mm4 = Y mask (FF00FF00FF00FF00)
;		ebx = line width in bytes (2*line width in pixels)
;		eax = line width in pixels
;	Modifies mm0, mm1, mm2, mm3
	
%macro YUYV_to_YUV9_ProcessY 0

	; Load the 8x2 block in MMX registers
	movq mm0, qword [esi]		; mm0 = 1st 4 pixels from 1st row
	movq mm1, qword [esi + 8]	; mm1 = 2nd 4 pixels from 1st row
	movq mm2, qword [esi + ebx]	; mm2 = 1st 4 pixels from 2nd row
	movq mm3, qword [esi + ebx + 8]	; mm3 = 2nd 4 pixels from 2nd row

	; Mask the Y field
	pand mm0, mm4
	pand mm1, mm4
	pand mm2, mm4
	pand mm3, mm4

	; Shift words to get the Y value
	psrlw mm0, 8
	psrlw mm1, 8
	psrlw mm2, 8
	psrlw mm3, 8

	; Pack the registers
	packuswb mm0, mm1
	packuswb mm2, mm3
	
	; Write down the result
 	movq qword [edi], mm0
	movq qword [edi + eax], mm2

	; 16 instructions
%endm

;	Read the Y component from an 8x2 pixel block and write it down
;	Assumes:
;		mm5 = UV mask (00FF00FF00FF00FF)
;		ebx = line width in bytes (2*line width in pixels)
;	Modifies mm0, mm1, mm2, mm3, edi, eax

%macro YUYV_to_YUV9_ProcessUV 1

	push ebx
	; Load the registers
	movq mm0, qword [esi + 8*%1]	  ; mm0 = %1 th 4 pixels from 1st row
	movq mm1, qword [esi + ebx + 8*%1]; mm1 = %1 th 4 pixels from 2nd row
	movq mm2, qword [esi + 2*ebx + 8*%1]; mm2 = %1 th 4 pixels from 3rd row
	lea ebx, [ebx+2*ebx]		  ; ebx *= 3 
	movq mm3, qword [esi + ebx + 8*%1]; mm3 = %1 th 4 pixels from 4th row
	
	; Mask the U and V fields
	pand mm0, mm5
	pand mm1, mm5
	pand mm2, mm5
	pand mm3, mm5

	; Sum the U and V fields
	paddw mm0, mm1
	paddw mm2, mm3
	paddw mm0, mm2

 	movq mm1, mm0
	psrlq mm1, 32
	paddw mm0, mm1
	
	movd eax, mm0

	; Divide by 8 to do the mean
	add eax, 00040004h
	shr eax, 3

	; Store the result
	mov edi, dword [YUV9_offsetV]
	mov byte [edi + %1], al
	mov edi, dword [YUV9_offsetU]	
	shr eax, 16
	mov byte [edi + %1], al	
	; Restore ebx
	pop ebx

	; 25 instructions
	
%endm
	
	
; void UYVY_to_YUV9(void * buffer_YUV9, void * buffer_UYVY, word width, word height)
align 16
YUYV_to_YUV9:

	push ebp
	mov ebp, esp
	push edi
	push esi
	push ebx
	push edx
	push ecx

	mov edi, dword [ebp + 8]		; edi = buffer_YUV9 
	mov esi, dword [ebp + 12]		; esi = buffer_UYVY

	movzx ecx, word [ebp + 20]		; ecx = height 

	; Compute YUV9_offsetU and YUV9_offsetV
	movzx eax, word [ebp + 16]		; eax = width 
	mul ecx					; eax = width * height 
	lea ebx, [edi + eax]			; ebx = buffer + width*height 
	mov dword [YUV9_offsetU], ebx		; YUV9_offsetU = ebx
	shr eax, 4				; eax = width * height / 16 
	add ebx, eax				; ebx = buffer + width*height*(1+1/16) 
	mov dword [YUV9_offsetV], ebx		; YUV9_offsetV = ebx

	movq mm4, qword [YUYV_maskY]	; Load the Y mask in mm4
	movq mm5, qword [YUYV_maskUV]   ; Load the UV mask in mm5 

	shr ecx, 2				; ecx = height / 4	
align 16
YUYV_to_YUV9_loop0:
	movzx edx, word [ebp + 16]
	shr edx, 3				; edx = width / 8 + 1
align 16
YUYV_to_YUV9_loop1:
	; We process 8x4-pixel blocks
	
	;-- Y --
	movzx ebx, word [ebp + 16]		; ebx = width	
	movzx eax, word [ebp + 16]		; eax = width
	shl ebx, 1				; ebx = width * 2
	
	; Process the top 8x2 block	
	YUYV_to_YUV9_ProcessY

	; Goto next two lines
   	add esi, ebx
	add edi, ebx
	add esi, ebx
		
	; Process the bottom 8x2 block
	YUYV_to_YUV9_ProcessY
	
	; Go back to first line
	sub esi, ebx
	sub edi, ebx
	sub esi, ebx

	;-- U & V --
	push edi
	YUYV_to_YUV9_ProcessUV 0
	YUYV_to_YUV9_ProcessUV 1
	pop edi

	; ~ 100 instructions
	
	add edi, 8
	add esi, 16
	add dword [YUV9_offsetU], 2
	add dword [YUV9_offsetV], 2

	dec edx
	or edx, edx
	jnz near YUYV_to_YUV9_loop1

	shr ebx, 1		; ebx = width	
	lea ebx, [ebx+2*ebx]	; ebx = 3*width
	add esi, ebx		; esi = (0, y+4)
	add edi, ebx		; edi = (0, y+4)
	add esi, ebx		; esi = (0, y+4)
	
	dec ecx
	cmp ecx, 0
	jnz near YUYV_to_YUV9_loop0

	pop ecx
	pop edx	
	pop ebx	
	pop esi
	pop edi
	pop ebp
	ret

; void YUYV_to_YUV12i(void * buffer_YUV12, void * buffer_YUYV, void * buffer_YUYV, word width, word height);
;

align 16
YUYV_to_YUV12i:

	push ebp
	mov ebp, esp
	push edi
	push esi
	push ebx
	push edx
	push ecx

	mov edi, dword [ebp + 8]		; edi = buffer_YUV12 
	mov esi, dword [ebp + 12]		; esi = buffer_YUYV1

	movzx ecx, word [ebp + 24]		; ecx = height 

	; Compute YUV12_offsetU and YUV12_offsetV
	movzx eax, word [ebp + 20]		; eax = width 
	mul ecx					; eax = width * height
	mov ebx, eax				; ebx = width * height
	mov dword [YUV12_offsetU], ebx		; YUV12_offsetU = width*height
	shr eax, 2			
	add ebx, eax				; ebx = 5 * width * height / 4 
	mov dword [YUV12_offsetV], ebx		; YUV12_offsetV = ebx

	movq mm4, qword [YUYV_maskY]		; Load the Y mask in mm4
	movq mm5, qword [YUYV_maskUV]		; Load the UV mask in mm5 

	shr ecx, 1				; ecx = height / 2	

	movzx edx, word [ebp + 20]
	shr edx, 2				
	mov ebx, edx
	shr edx, 1				; edx = width / 8

	push edx				; store this value 
	mov ebp, dword [ebp + 16]		; ebp = buffer_YUYV2

align 16
.loop0:
	pop edx
	push edx
align 16
.loop1:
	; We process 8x2-pixel blocks

	;; TODO

	;-- Y --
	movq mm0, qword [esi]
	movq mm1, qword [esi + 8]
	movq mm2, qword [ebp]
	movq mm3, qword [ebp + 8]
	pand mm0, mm4
	pand mm1, mm4
	pand mm2, mm4
	pand mm3, mm4	
	psrlw mm0, 8
	psrlw mm1, 8
	psrlw mm2, 8
	psrlw mm3, 8
	packuswb mm0, mm1
	packuswb mm2, mm3
	movq qword [edi], mm0		; write the pixels
	movq qword [edi + 4*ebx], mm2	; write the pixel below

	;-- U --	
	;-- V --
	movq mm0, qword [esi]		  ; MM0: YUYVYUYV 
	movq mm1, qword [esi + 8]	  
	movq mm2, qword [ebp]
	movq mm3, qword [ebp + 8]
	pand mm0, mm5			  ; MM0: 0U0V0U0V 
	pand mm1, mm5
	pand mm2, mm5
	pand mm3, mm5	

	; Sum the U and V fields
	paddw mm0, mm2			  
	paddw mm1, mm3

	psrlw mm0, 1			  ; MM0: 0U0V0U0V (mean of two lines)
	psrlw mm1, 1
	packuswb mm0, mm1		  ; MM0: UVUVUVUV
	movq mm1, mm0
	
	pand mm0, mm4			  ; MM0: U0U0U0U0 
	pand mm1, mm5			  ; MM1: 0V0V0V0V 
	psrlw mm0, 8			  ; MM0: 0U0U0U0U 

	packuswb mm0, mm0		  ; MM0: ----UUUU
	packuswb mm1, mm1		  ; MM1: ----VVVV

 	add edi, [YUV12_offsetU]
	movd [edi], mm0
	sub edi, [YUV12_offsetU]
	sub dword [YUV12_offsetU], 4	  ; Adjust offset

	add edi, [YUV12_offsetV]
	movd [edi], mm1
	sub edi, [YUV12_offsetV]
	sub dword [YUV12_offsetV], 4	  ; Adjust offset
		
	add edi, 8			; increase the counter
	add esi, 16			; increase the counter
	add ebp, 16			; increase the counter

	dec edx
	or edx, edx
	jnz near .loop1

	shl ebx, 2
	lea edi, [edi + ebx]
	sub [YUV12_offsetU], ebx	  ; Adjust offset
	sub [YUV12_offsetV], ebx	  ; Adjust offset
	shr ebx, 2
	
	dec ecx
	or ecx, ecx
	jnz near .loop0

	pop edx
	
	pop ecx
	pop edx	
	pop ebx	
	pop esi
	pop edi
	pop ebp
	ret

; void UYVV_to_YUV12i(void * buffer_YUV12, void * buffer_UYVY, void * buffer_UYVY, word width, word height);
;

align 16
UYVY_to_YUV12i:

	push ebp
	mov ebp, esp
	push edi
	push esi
	push ebx
	push edx
	push ecx

	mov edi, dword [ebp + 8]		; edi = buffer_YUV12 
	mov esi, dword [ebp + 12]		; esi = buffer_UYVY1

	movzx ecx, word [ebp + 24]		; ecx = height 

	; Compute YUV12_offsetU and YUV12_offsetV
	movzx eax, word [ebp + 20]		; eax = width 
	mul ecx					; eax = width * height
	mov ebx, eax				; ebx = width * height
	mov dword [YUV12_offsetU], ebx		; YUV12_offsetU = width*height
	shr eax, 2			
	add ebx, eax				; ebx = 5 * width * height / 4 
	mov dword [YUV12_offsetV], ebx		; YUV12_offsetV = ebx

	movq mm4, qword [UYVY_maskY]		; Load the Y mask in mm4
	movq mm5, qword [UYVY_maskUV]		; Load the UV mask in mm5 

	shr ecx, 1				; ecx = height / 2	

	movzx edx, word [ebp + 20]
	shr edx, 2				
	mov ebx, edx
	shr edx, 1				; edx = width / 8

	push edx				; store this value 
	mov ebp, dword [ebp + 16]		; ebp = buffer_UYVY2

align 16
.loop0:
	pop edx
	push edx
align 16
.loop1:
	; We process 8x2-pixel blocks

	;-- Y --
	movq mm0, qword [esi]
	movq mm1, qword [esi + 8]
	movq mm2, qword [ebp]
	movq mm3, qword [ebp + 8]
	pand mm0, mm4
	pand mm1, mm4
	pand mm2, mm4
	pand mm3, mm4	
	packuswb mm0, mm1
	packuswb mm2, mm3
	movq qword [edi], mm0		; write the pixels
	movq qword [edi + 4*ebx], mm2	; write the pixel below

	;-- U --	
	;-- V --
	movq mm0, qword [esi]		  ; MM0: UYVYUYVY 
	movq mm1, qword [esi + 8]	  
	movq mm2, qword [ebp]
	movq mm3, qword [ebp + 8]
	pand mm0, mm5			  ; MM0: U0V0U0V0 
	pand mm1, mm5
	pand mm2, mm5
	pand mm3, mm5	
	psrlw mm0, 8
	psrlw mm1, 8
	psrlw mm2, 8
	psrlw mm3, 8

	; Sum the U and V fields
	paddw mm0, mm2			  
	paddw mm1, mm3

	psrlw mm0, 1			  ; MM0: 0U0V0U0V (mean of two lines)
	psrlw mm1, 1
	packuswb mm0, mm1		  ; MM0: UVUVUVUV
	movq mm1, mm0
	
	pand mm0, mm5			  ; MM0: U0U0U0U0 
	pand mm1, mm4			  ; MM1: 0V0V0V0V 
	psrlw mm0, 8			  ; MM0: 0U0U0U0U 

	packuswb mm0, mm0		  ; MM0: ----UUUU
	packuswb mm1, mm1		  ; MM1: ----VVVV

 	add edi, [YUV12_offsetU]
	movd [edi], mm0
	sub edi, [YUV12_offsetU]
	sub dword [YUV12_offsetU], 4	  ; Adjust offset

	add edi, [YUV12_offsetV]
	movd [edi], mm1
	sub edi, [YUV12_offsetV]
	sub dword [YUV12_offsetV], 4	  ; Adjust offset
		
	add edi, 8			; increase the counter
	add esi, 16			; increase the counter
	add ebp, 16			; increase the counter

	dec edx
	or edx, edx
	jnz near .loop1

	shl ebx, 2
	lea edi, [edi + ebx]
	sub [YUV12_offsetU], ebx	  ; Adjust offset
	sub [YUV12_offsetV], ebx	  ; Adjust offset
	shr ebx, 2
	
	dec ecx
	or ecx, ecx
	jnz near .loop0

	pop edx
	
	pop ecx
	pop edx	
	pop ebx	
	pop esi
	pop edi
	pop ebp
	ret

END

	


