#ifndef __RTP_H
#define __RTP_H

#define RTP_PT_MPA  14
#define RTP_PT_MPV  32
#define RTP_PT_MP2T 33

/* From MPEG2-TS */
struct rtp_bits {
  unsigned int v:2;           /* version: 2 */
  unsigned int p:1;           /* is there padding appended: 0 */
  unsigned int x:1;           /* number of extension headers: 0 */
  unsigned int cc:4;          /* number of CSRC identifiers: 0 */
  unsigned int m:1;           /* marker: 0 */
  unsigned int pt:7;          /* payload type: 14 for MPEG audio */
  unsigned int sequence:16;   /* sequence number: random */
};

struct rtp_header
{
  struct rtp_bits b;
  int timestamp;
  int ssrc;

  rtp_header();
};

struct rtp_mpeg_video_specific_header : public rtp_header
{
  struct _video_
  {
    unsigned int mbz:5;  /* zero */
    unsigned int t:1;    /* MPEG2 */
    unsigned int tr:10;  /* temporal reference */
    unsigned int an:1;   /* active n bit */
    unsigned int n:1;    /* new picture header */
    unsigned int s:1;    /* sequence header */
    unsigned int b:1;    /* beginning of slice */
    unsigned int e:1;    /* end of slice */
    unsigned int p:3;    /* picture type */
    unsigned int fbv:1;  /* full_pel_backward_vector */
    unsigned int bfc:3;  /* backward_f_code */
    unsigned int ffv:1;  /* full_pel_forward_vector */
    unsigned int ffc:3;  /* forward_f_code */
  } video;

  rtp_mpeg_video_specific_header();
};

struct rtp_mpeg2_video_specific_header : public rtp_mpeg_video_specific_header
{
  struct _video_extension_
  {
    unsigned int x:1;    /* zero */
    unsigned int e:1;    /* extension */
    unsigned int f_00:4; /* forward horizontal f_code */
    unsigned int f_01:4; /* forward vertical f_code */
    unsigned int f_10:4; /* backward horizontal f_code */
    unsigned int f_11:4; /* backward vertical f_code */
    unsigned int dc:2;   /* intra_DC_precision */
    unsigned int ps:2;   /* picture_structure */
    unsigned int t:1;    /* top_field_first */
    unsigned int p:1;    /* frame_predicted_frame_dct */
    unsigned int c:1;    /* concealment_motion_vectors */
    unsigned int q:1;    /* q_scale type */
    unsigned int v:1;    /* intra_vlc_format */
    unsigned int a:1;    /* alternate scan */
    unsigned int r:1;    /* repeat_first_field */
    unsigned int h:1;    /* chroma_420_type */
    unsigned int g:1;    /* progressive frame */
    unsigned int d:1;    /* composite_display_flag */
  } video_extension;
  
  rtp_mpeg2_video_specific_header();
};

struct rtp_mpeg_audio_specific_header : public rtp_header
{    
  struct
  {
    unsigned short int mbz;         /* zero */
    unsigned short int frag_offset; /* Byte offset into the audio frame for the data in this packet. */
  } audio;
  
  rtp_mpeg_audio_specific_header();
};

#endif
