/* 
 *    sockmcast.cpp
 *
 *	Copyright (C) Vivien Chappelier - 2000
 *
 *  This file is part of fame, a free MPEG encoder.
 *	
 *  fame is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  fame is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#include "sockmcast.h"

sockmcastbuf::~sockmcastbuf()
{
  if(membership)
    setopt((sockbuf::option) IP_DROP_MEMBERSHIP, (char *) & mreq, sizeof(mreq), (sockbuf::level) IPPROTO_IP);
}

int sockmcastbuf::bind (char const * address, int port)
{
  sockinetaddr sa(address, port);

  return(bind(sa));
}

int sockmcastbuf::bind (int address, int port)
{
  sockinetaddr sa(address, port);

  return(bind(sa));
}

int sockmcastbuf::bind (sockAddr& sa)
{
  if(is_address_multicast(sa))
  {
    // Register from a multicast address
    mreq.imr_multiaddr.s_addr = ((struct sockaddr_in *) sa.addr())->sin_addr.s_addr;
    mreq.imr_interface.s_addr = INADDR_ANY;

    setopt((sockbuf::option) IP_ADD_MEMBERSHIP, (char *) & mreq, sizeof(mreq), (sockbuf::level) IPPROTO_IP);
    membership = true;
  }

  return sockinetbuf::bind (sa);
}

bool sockmcastbuf::is_address_multicast(sockAddr const & sa)
{
  long addr = ((struct sockaddr_in *) sa.addr())->sin_addr.s_addr;

  return(((addr & 255) >= 224 && (addr & 255) <= 239));
}

isockmcast::isockmcast (int proto)
  : ios (new sockmcastbuf (sockbuf::sock_dgram, proto))
{}

isockmcast::isockmcast (const sockbuf& sb)
  : ios (new sockmcastbuf (sb))
{}

osockmcast::osockmcast (int proto)
  : ios (new sockmcastbuf (sockbuf::sock_dgram, proto))
{}

osockmcast::osockmcast (const sockbuf& sb)
  : ios (new sockmcastbuf (sb))
{}

iosockmcast::iosockmcast (int proto)
  : ios (new sockmcastbuf (sockbuf::sock_dgram, proto))
{}

iosockmcast::iosockmcast (const sockbuf& sb)
  : ios (new sockmcastbuf (sb))
{}
