/* 
 *    mmx_coder.h
 *
 *	Copyright (C) Vivien Chappelier - 2000
 *
 *  This file is part of fame, a free MPEG encoder.
 *	
 *  fame is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  fame is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#ifndef __MMX_CODER_H
#define __MMX_CODER_H

#include <iostream.h>

#include <common.h>
#include <frame.h>
#include <io/bitio.h>

#define MODULE_NAME "mmx"

#define ABS(x) (((x)>0)?(x):(-(x)))

#define SEQUENCE_HEADCODE 0x01b3
#define SEQUENCE_ENDCODE  0x01b9
#define GOP_STARTCODE 0x1b8
#define PICT_STARTCODE 0x100
#define SLICE_BASE_CODE 0x100

/* The frame type values */
#define I_FRAME 1
#define P_FRAME 2
#define B_FRAME 3

/* Perform the Dicrete Cosine Tranform on a 8x8 pixel block */
#define LG2_DCTSIZE 3
#define DCTSIZE (1L << LG2_DCTSIZE)

/* Default values */
#define DEFAULT_QSCALE 8
#define DEFAULT_SLICE_LENGTH 2048
#define DEFAULT_NUMBER_OF_FRAMES 0
#define DEFAULT_START_CODE_FREQUENCY 100

typedef int16 Block[DCTSIZE][DCTSIZE];

class MMXMpegVideoCoder {
public:
        MMXMpegVideoCoder(int16 qscale_, dword slice_length_, dword number_of_frames_, dword start_code_frequency);
        ~MMXMpegVideoCoder();

	void StartSequence(word width, word height);
	void EndSequence();

        int32 EncodeFrame(VideoFrame const & frame);

	void SetCallback(video_callback_f callback);
private:
	Bitbuffer * buffer;
	
	int16 qscale;
	dword slice_length;
	dword number_of_frames;
	dword start_code_frequency;
	bool  verbose;
	int16 qmatrix[64];
	Block block;
	int32 y_dc_pred, cr_dc_pred, cb_dc_pred;
	dword frame_number;
	video_callback_f callback;
};

char module_id[] = MODULE_NAME;
extern "C" MMXMpegVideoCoder * module_init(int argc, char * argv[]);
extern "C" void module_close(MMXMpegVideoCoder * coder);
extern "C" void video_encode_frame(MMXMpegVideoCoder * context, VideoFrame * frame);

#endif
